#  Copyright (C) 2021
#  ABM, Moscow
#
#  UNPUBLISHED PROPRIETARY MATERIAL.
#  ALL RIGHTS RESERVED.
#
#  Authors: Mike Orlov <m.orlov@abm-jsc.ru>
#
from ._cast import *
from ._conditional import *
from ._collection import *
from ._dataclass import *
from ._exception import *
from ._file import *
from ._hint import *
from ._ini import *
from ._init import *
from ._instance import *
from ._json import *
from ._toml import *
from ._type import *
from ._utils import *
from .no_value import NoValue



#  Copyright (C) 2026
#  ABM, Moscow
#
#  UNPUBLISHED PROPRIETARY MATERIAL.
#  ALL RIGHTS RESERVED.
#
#  Authors: Mike Orlov <m.orlov@abm-jsc.ru>
from ._exceptions import ParseError, IniParseError
from .make_ini_from_hint import make_ini_from_hint
from .parse_ini_content import parse_ini_content
from .read_ini_file_by_path import read_ini_file_by_path



#  Copyright (C) 2026
#  ABM, Moscow
#
#  UNPUBLISHED PROPRIETARY MATERIAL.
#  ALL RIGHTS RESERVED.
#
#  Authors: Mike Orlov <m.orlov@abm-jsc.ru>
import datetime
from decimal import Decimal
from typing import TypeAlias, Union

from .repr_in_dumps import ReprInDumps
from .._type import DataclassProtocol

JsonLoaded: TypeAlias = Union[list['JsonLoaded'], dict[str, 'JsonLoaded'], str, int, float, None]
Jsonable: TypeAlias = Union[JsonLoaded, list['Jsonable'], tuple['Jsonable', ...], dict[str | int, 'Jsonable']]
CustomJsonable: TypeAlias = Union[
    Jsonable, list['CustomJsonable'], tuple['CustomJsonable', ...], dict[str | int, 'CustomJsonable'],
    set['CustomJsonable'], frozenset['CustomJsonable'],
    bytes, ReprInDumps, type, DataclassProtocol, datetime.datetime, datetime.date, datetime.time, Decimal, Exception
]



#  Copyright (C) 2026
#  ABM, Moscow
#
#  UNPUBLISHED PROPRIETARY MATERIAL.
#  ALL RIGHTS RESERVED.
#
#  Authors: Mike Orlov <m.orlov@abm-jsc.ru>
import dataclasses

from .structure_hint import StructureHint
from .field_hint import FieldHint
from .._dataclass import get_dataclass_field_name_to_field


def get_hint_from_dataclass(dataclass_type: type) -> StructureHint:
    if not dataclasses.is_dataclass(dataclass_type):
        raise TypeError("expected dataclass type")

    args = []
    dataclass_fields = get_dataclass_field_name_to_field(dataclass_type)
    for field_name in dataclass_fields:
        field = dataclass_fields[field_name]
        if not field.init:
            continue

        default = dataclasses.MISSING
        if field.default != dataclasses.MISSING:
            default = field.default
        elif field.default_factory != dataclasses.MISSING:
            default = field.default_factory

        field_type = field.type
        if dataclasses.is_dataclass(field_type):
            hint = get_hint_from_dataclass(field_type)
            hint = dataclasses.replace(hint, name=field_name)
        else:
            hint = FieldHint(field_name, field_type, default)

        args.append(hint)
    return StructureHint(args)



#  Copyright (C) 2024
#  ABM, Moscow
#
#  UNPUBLISHED PROPRIETARY MATERIAL.
#  ALL RIGHTS RESERVED.
#
#  Authors: Mike Orlov <m.orlov@abm-jsc.ru>
# import dataclasses
import typing


class PickableDict(dict):
    def pick(self, keys: typing.Iterable) -> dict:
        """Returns new dict with picked keys only"""
        return {k: self[k] for k in keys if k in self}
